function GBMOptionVal = optionValueGBM(prodprofile,sIndex, oilP0, vol, r, lifetime, divYield, nSteps,...
                                        developT,researchLag,developCost,Abex,expirationT)           
%optionValueGBM computes the option value using a geometric brownian motion
%price process. It furthermore uses a binomial approach to compute the end
%node, and then rolls back the tree to find the the value of the option

%-------------------------------CAPEX------------------------------
%properly discount the capex and make it into one value, the exercise price
totCapex=0;

    if sIndex>1

        lag = researchLag; 
    else
        lag=0;
    end 

for i= 1: developT 
   totCapex=totCapex+developCost(sIndex)/((1+r)^(i+lag-0.5));
end

 GBMOption=NaN(nSteps*6+3,nSteps+1);

%---------------------up &down factors-------------------------------------
    %define time length for each time node
    h= expirationT/nSteps;
    %compute up and down factors
    up = exp(vol * h ^ 0.5);
    down = exp(-vol * h ^ 0.5);
    %calculate risk neutral probabilities
    rhat = exp(r * h);
    dhat = exp(divYield * h);
    ahat = rhat / dhat; %e^(r-d)h
    %compute up and down probabilities
    p_up = (ahat - down) / (up - down); %p_u= [e^(r-d)h)-d ]/u-d
    p_down = 1 - p_up;
%--------------------------------------------------------------------------


%-----------------Initial array of terminal stock and option values-------
j=1;
for i= 1:nSteps+1
    
   %Compute oil prices in terminal nodes
   oilP(i)=oilP0*(up^(nSteps+1-i))*(down^(i-1));
   
   %reset project value for each node
    projectVal(i)=0;
   
    %iterate through the lifetime of the proejct and compute the NPV 
   for u=1:length(prodprofile) 
         %calculate projectvalue if project initiated at node
        %Takes developT years to develop the field, so the production starts after developT time
        %Expected prices used as the decision is made and is irreversible. 
        %Note that expected prices are p0*exp(u+developT) because of development time of the field.
      NPV=(oilP(i)*exp((r-divYield)*(u+developT))*prodprofile(u) - (prodprofile(u) ...
           * 46.34 + 605.63)) / exp(r * (u +developT +lag - 0.5));
       
       if (NPV < 0)
            projectVal(i) = projectVal(i) - Abex / exp(r * (u+developT +lag - 0.5));
            
        break
       end
       if (u == length(prodprofile))
            projectVal(i) = projectVal(i) - Abex / exp(r * (u +developT + lag- 0.5));
       end
              
       projectVal(i) = projectVal(i) + NPV;
   end
   
   W(i) = max(projectVal(i) - totCapex, 0);
   
   %code to create matrix that produces the binomial tree
   
   GBMOption(j,nSteps+1)=oilP(i);
   GBMOption(j+1,nSteps+1)=projectVal(i);
   GBMOption(j+2,nSteps+1)=W(i);
   j=j+6;
end
   


%option value is max off exercising now or never



%-------------------Iterate through the rest of the binomial tree----------

for ts=nSteps:-1:1
    j= (nSteps+1-ts)*3+1 ;
    %calculate next column stock prices and option values
    for i = 1:ts
        %stock at ts N node divided by up movement to find current stock price
        oilPNew(i) = oilP(i) / up;
        
        projectValNew(i) = 0;
        %ProjectValNew(i) = ProjectVal(i) / up
   
        for u = 1:length(prodprofile)  %Lifetime production years, but first is at year 1
            
            %calculate projectvalue if project initiated at node
            opex(u)=(prodprofile(u) * 46.34 + 605.63);
            NPV = (oilPNew(i)*exp((r-divYield)*(u + developT)) * prodprofile(u) - opex(u)) / exp(r * (u+ developT + lag - 0.5));
                
            if (NPV < 0)
            projectValNew(i) = projectValNew(i) - Abex / exp(r * (u+developT + lag - 0.5));
                
            break 
            end
            
            projectValNew(i) = projectValNew(i) + NPV;
            
            if (u == length(lifetime))
            projectValNew(i) = projectValNew(i) - Abex / exp(r * (u +2 + lag- 0.5));
            
            end
        end
        %Option value is Max off value of exercising now and  value of waiting
        WNew(i) = max((p_up * W(i) + p_down * W(i + 1)) / rhat, projectValNew(i) - totCapex);        
       
        %update option value and oil price for each node
        W(i) = WNew(i);
        oilP(i) = oilPNew(i);
        %projectVal(i) = ProjectValNew(i);
        
        
   %code to create matrix that produces the binomial tree     
   GBMOption(j,ts)=oilP(i);
   GBMOption(j+1,ts)=projectValNew(i);
   GBMOption(j+2,ts)=WNew(i);
   j=j+6;
    end

end
%------------------------------------------------------------------------------   
%returns a single option value for one case in a certain scenario
GBMOptionVal = WNew(1);


end


